'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _temp2;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _CellAction = require('./CellAction');

var _CellAction2 = _interopRequireDefault(_CellAction);

var _CellExpand = require('./CellExpand');

var _CellExpand2 = _interopRequireDefault(_CellExpand);

var _ChildRowDeleteButton = require('./ChildRowDeleteButton');

var _ChildRowDeleteButton2 = _interopRequireDefault(_ChildRowDeleteButton);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var React = require('react');

var joinClasses = require('classnames');
var EditorContainer = require('./editors/EditorContainer');
var ExcelColumn = require('./PropTypeShapes/ExcelColumn');
var isFunction = require('./utils/isFunction');
var CellMetaDataShape = require('./PropTypeShapes/CellMetaDataShape');
var SimpleCellFormatter = require('./formatters/SimpleCellFormatter');
var ColumnUtils = require('./ColumnUtils');
var createObjectWithProperties = require('./createObjectWithProperties');

require('./themes/react-data-grid-cell.css');

// The list of the propTypes that we want to include in the Cell div
var knownDivPropertyKeys = ['height', 'tabIndex', 'value'];

var Cell = (_temp2 = _class = function (_React$Component) {
  _inherits(Cell, _React$Component);

  function Cell() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, Cell);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = Cell.__proto__ || Object.getPrototypeOf(Cell)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      isCellValueChanging: false,
      isLockChanging: false
    }, _this.onCellClick = function (e) {
      var meta = _this.props.cellMetaData;
      if (meta != null && meta.onCellClick && typeof meta.onCellClick === 'function') {
        meta.onCellClick({ rowIdx: _this.props.rowIdx, idx: _this.props.idx }, e);
      }
    }, _this.onCellFocus = function () {
      var meta = _this.props.cellMetaData;
      if (meta != null && meta.onCellFocus && typeof meta.onCellFocus === 'function') {
        meta.onCellFocus({ rowIdx: _this.props.rowIdx, idx: _this.props.idx });
      }
    }, _this.onCellContextMenu = function () {
      var meta = _this.props.cellMetaData;
      if (meta != null && meta.onCellContextMenu && typeof meta.onCellContextMenu === 'function') {
        meta.onCellContextMenu({ rowIdx: _this.props.rowIdx, idx: _this.props.idx });
      }
    }, _this.onCellDoubleClick = function (e) {
      var meta = _this.props.cellMetaData;
      if (meta != null && meta.onCellDoubleClick && typeof meta.onCellDoubleClick === 'function') {
        meta.onCellDoubleClick({ rowIdx: _this.props.rowIdx, idx: _this.props.idx }, e);
      }
    }, _this.onCellExpand = function (e) {
      e.stopPropagation();
      var meta = _this.props.cellMetaData;
      if (meta != null && meta.onCellExpand != null) {
        meta.onCellExpand({ rowIdx: _this.props.rowIdx, idx: _this.props.idx, rowData: _this.props.rowData, expandArgs: _this.props.expandableOptions });
      }
    }, _this.onCellKeyDown = function (e) {
      if (_this.canExpand() && e.key === 'Enter') {
        _this.onCellExpand(e);
      }
    }, _this.onDeleteSubRow = function () {
      var meta = _this.props.cellMetaData;
      if (meta != null && meta.onDeleteSubRow != null) {
        meta.onDeleteSubRow({ rowIdx: _this.props.rowIdx, idx: _this.props.idx, rowData: _this.props.rowData, expandArgs: _this.props.expandableOptions });
      }
    }, _this.onDragHandleDoubleClick = function (e) {
      e.stopPropagation();
      var meta = _this.props.cellMetaData;
      if (meta != null && meta.onDragHandleDoubleClick && typeof meta.onDragHandleDoubleClick === 'function') {
        meta.onDragHandleDoubleClick({ rowIdx: _this.props.rowIdx, idx: _this.props.idx, rowData: _this.getRowData(), e: e });
      }
    }, _this.onDragOver = function (e) {
      e.preventDefault();
    }, _this.getStyle = function () {
      var style = {
        position: 'absolute',
        width: _this.props.column.width,
        height: _this.props.height,
        left: _this.props.column.left,
        contain: 'layout'
      };
      return style;
    }, _this.getFormatter = function () {
      var col = _this.props.column;
      if (_this.isActive()) {
        return React.createElement(EditorContainer, { rowData: _this.getRowData(), rowIdx: _this.props.rowIdx, value: _this.props.value, idx: _this.props.idx, cellMetaData: _this.props.cellMetaData, column: col, height: _this.props.height });
      }

      return _this.props.column.formatter;
    }, _this.getRowData = function () {
      var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.props;

      return props.rowData.toJSON ? props.rowData.toJSON() : props.rowData;
    }, _this.getFormatterDependencies = function () {
      // convention based method to get corresponding Id or Name of any Name or Id property
      if (typeof _this.props.column.getRowMetaData === 'function') {
        return _this.props.column.getRowMetaData(_this.getRowData(), _this.props.column);
      }
    }, _this.getCellClass = function () {
      var className = joinClasses(_this.props.column.cellClass, 'react-grid-Cell', _this.props.className, _this.props.column.locked ? 'react-grid-Cell--locked' : null);
      var extraClasses = joinClasses({
        'row-selected': _this.props.isRowSelected,
        editing: _this.isActive(),
        copied: _this.isCopied() || _this.wasDraggedOver() || _this.isDraggedOverUpwards() || _this.isDraggedOverDownwards(),
        'is-dragged-over-up': _this.isDraggedOverUpwards(),
        'is-dragged-over-down': _this.isDraggedOverDownwards(),
        'was-dragged-over': _this.wasDraggedOver(),
        'cell-tooltip': _this.props.tooltip ? true : false,
        'rdg-child-cell': _this.props.expandableOptions && _this.props.expandableOptions.subRowDetails && _this.props.expandableOptions.treeDepth > 0,
        'last-column': _this.props.column.isLastColumn
      });
      return joinClasses(className, extraClasses);
    }, _this.getUpdateCellClass = function () {
      return _this.props.column.getUpdateCellClass ? _this.props.column.getUpdateCellClass(_this.props.selectedColumn, _this.props.column, _this.state.isCellValueChanging) : '';
    }, _this.isColumnSelected = function () {
      var meta = _this.props.cellMetaData;
      if (meta == null) {
        return false;
      }

      return meta.selected && meta.selected.idx === _this.props.idx;
    }, _this.isSelected = function () {
      var meta = _this.props.cellMetaData;
      if (meta == null) {
        return false;
      }

      return meta.selected && meta.selected.rowIdx === _this.props.rowIdx && meta.selected.idx === _this.props.idx;
    }, _this.isActive = function () {
      var meta = _this.props.cellMetaData;
      if (meta == null) {
        return false;
      }
      return _this.isSelected() && meta.selected.active === true;
    }, _this.isCellSelectionChanging = function (nextProps) {
      var meta = _this.props.cellMetaData;
      if (meta == null) {
        return false;
      }
      var nextSelected = nextProps.cellMetaData.selected;
      if (meta.selected && nextSelected) {
        return _this.props.idx === nextSelected.idx || _this.props.idx === meta.selected.idx;
      }

      return true;
    }, _this.isCellSelectEnabled = function () {
      var meta = _this.props.cellMetaData;
      if (meta == null) {
        return false;
      }
      return meta.enableCellSelect;
    }, _this.hasChangedDependentValues = function (nextProps) {
      var currentColumn = _this.props.column;
      var hasChangedDependentValues = false;

      if (currentColumn.getRowMetaData) {
        var currentRowMetaData = currentColumn.getRowMetaData(_this.getRowData(), currentColumn);
        var nextColumn = nextProps.column;
        var nextRowMetaData = nextColumn.getRowMetaData(_this.getRowData(nextProps), nextColumn);

        hasChangedDependentValues = !_lodash2['default'].isEqual(currentRowMetaData, nextRowMetaData);
      }

      return hasChangedDependentValues;
    }, _this.applyUpdateClass = function () {
      var updateCellClass = _this.getUpdateCellClass();
      // -> removing the class
      if (updateCellClass != null && updateCellClass !== '') {
        var cellDOMNode = _this.node;
        if (cellDOMNode.classList) {
          cellDOMNode.classList.remove(updateCellClass);
          // -> and re-adding the class
          cellDOMNode.classList.add(updateCellClass);
        } else if (cellDOMNode.className.indexOf(updateCellClass) === -1) {
          // IE9 doesn't support classList, nor (I think) altering element.className
          // without replacing it wholesale.
          cellDOMNode.className = cellDOMNode.className + ' ' + updateCellClass;
        }
      }
    }, _this.setScrollLeft = function (scrollLeft) {
      var node = _this.node;
      if (node) {
        var transform = 'translate3d(' + scrollLeft + 'px, 0px, 0px)';
        node.style.webkitTransform = transform;
        node.style.transform = transform;
      }
    }, _this.removeScroll = function () {
      var node = _this.node;
      if (node) {
        node.style.webkitTransform = null;
        node.style.transform = null;
      }
    }, _this.isCopied = function () {
      var copied = _this.props.cellMetaData.copied;
      return copied && copied.rowIdx === _this.props.rowIdx && copied.idx === _this.props.idx;
    }, _this.isDraggedOver = function () {
      var dragged = _this.props.cellMetaData.dragged;
      return dragged && dragged.overRowIdx === _this.props.rowIdx && dragged.idx === _this.props.idx;
    }, _this.wasDraggedOver = function () {
      var dragged = _this.props.cellMetaData.dragged;
      return dragged && (dragged.overRowIdx < _this.props.rowIdx && _this.props.rowIdx < dragged.rowIdx || dragged.overRowIdx > _this.props.rowIdx && _this.props.rowIdx > dragged.rowIdx) && dragged.idx === _this.props.idx;
    }, _this.isDraggedCellChanging = function (nextProps) {
      var isChanging = void 0;
      var dragged = _this.props.cellMetaData.dragged;
      var nextDragged = nextProps.cellMetaData.dragged;
      if (dragged) {
        isChanging = nextDragged && _this.props.idx === nextDragged.idx || dragged && _this.props.idx === dragged.idx;
        return isChanging;
      }

      return false;
    }, _this.isCopyCellChanging = function (nextProps) {
      var isChanging = void 0;
      var copied = _this.props.cellMetaData.copied;
      var nextCopied = nextProps.cellMetaData.copied;
      if (copied) {
        isChanging = nextCopied && _this.props.idx === nextCopied.idx || copied && _this.props.idx === copied.idx;
        return isChanging;
      }
      return false;
    }, _this.isDraggedOverUpwards = function () {
      var dragged = _this.props.cellMetaData.dragged;
      return !_this.isSelected() && _this.isDraggedOver() && _this.props.rowIdx < dragged.rowIdx;
    }, _this.isDraggedOverDownwards = function () {
      var dragged = _this.props.cellMetaData.dragged;
      return !_this.isSelected() && _this.isDraggedOver() && _this.props.rowIdx > dragged.rowIdx;
    }, _this.isFocusedOnBody = function () {
      return document.activeElement == null || document.activeElement.nodeName && typeof document.activeElement.nodeName === 'string' && document.activeElement.nodeName.toLowerCase() === 'body';
    }, _this.isFocusedOnCell = function () {
      return document.activeElement && document.activeElement.className.indexOf('react-grid-Cell') !== -1;
    }, _this.checkFocus = function () {
      if (_this.isSelected() && !_this.isActive()) {
        if (_this.props.isScrolling && !_this.props.cellMetaData.isScrollingVerticallyWithKeyboard && !_this.props.cellMetaData.isScrollingHorizontallyWithKeyboard) {
          return;
        }
        // If the enableCellAutoFocus is set in the ReactDataGrid props, it will allow the cell to take focus when the browser is focused on the body.
        // Otherwise, only focus to the current cell if the currently active node in the document is within the data grid.
        // Meaning focus should not be stolen from elements that the grid doesnt control.
        var cellAutoFocusEnabled = _this.props.cellMetaData && _this.props.cellMetaData.enableCellAutoFocus;
        var dataGridDOMNode = _this.props.cellMetaData && _this.props.cellMetaData.getDataGridDOMNode ? _this.props.cellMetaData.getDataGridDOMNode() : null;
        if (_this.isFocusedOnCell() || cellAutoFocusEnabled && _this.isFocusedOnBody() || dataGridDOMNode && dataGridDOMNode.contains(document.activeElement)) {
          var cellDOMNode = _this.node;
          if (cellDOMNode) {
            cellDOMNode.focus();
          }
        }
      }
    }, _this.canEdit = function () {
      return _this.props.column.editor != null || _this.props.column.editable;
    }, _this.canExpand = function () {
      return _this.props.expandableOptions && _this.props.expandableOptions.canExpand;
    }, _this.createColumEventCallBack = function (onColumnEvent, info) {
      return function (e) {
        onColumnEvent(e, info);
      };
    }, _this.createCellEventCallBack = function (gridEvent, columnEvent) {
      return function (e) {
        gridEvent(e);
        columnEvent(e);
      };
    }, _this.createEventDTO = function (gridEvents, columnEvents, onColumnEvent) {
      var allEvents = Object.assign({}, gridEvents);

      for (var eventKey in columnEvents) {
        if (columnEvents.hasOwnProperty(eventKey)) {
          var event = columnEvents[event];
          var eventInfo = { idx: _this.props.idx, rowIdx: _this.props.rowIdx, rowId: _this.props.rowData[_this.props.cellMetaData.rowKey], name: eventKey };
          var eventCallback = _this.createColumEventCallBack(onColumnEvent, eventInfo);

          if (allEvents.hasOwnProperty(eventKey)) {
            var currentEvent = allEvents[eventKey];
            allEvents[eventKey] = _this.createCellEventCallBack(currentEvent, eventCallback);
          } else {
            allEvents[eventKey] = eventCallback;
          }
        }
      }

      return allEvents;
    }, _this.getEvents = function () {
      var columnEvents = _this.props.column ? Object.assign({}, _this.props.column.events) : undefined;
      var onColumnEvent = _this.props.cellMetaData ? _this.props.cellMetaData.onColumnEvent : undefined;
      var gridEvents = {
        onClick: _this.onCellClick,
        onFocus: _this.onCellFocus,
        onDoubleClick: _this.onCellDoubleClick,
        onContextMenu: _this.onCellContextMenu,
        onDragOver: _this.onDragOver
      };

      if (!columnEvents || !onColumnEvent) {
        return gridEvents;
      }

      return _this.createEventDTO(gridEvents, columnEvents, onColumnEvent);
    }, _this.getKnownDivProps = function () {
      return createObjectWithProperties(_this.props, knownDivPropertyKeys);
    }, _this.renderCellContent = function (props) {
      var CellContent = void 0;
      var Formatter = _this.getFormatter();
      if (React.isValidElement(Formatter)) {
        props.dependentValues = _this.getFormatterDependencies();
        CellContent = React.cloneElement(Formatter, props);
      } else if (isFunction(Formatter)) {
        CellContent = React.createElement(Formatter, { value: _this.props.value, dependentValues: _this.getFormatterDependencies() });
      } else {
        CellContent = React.createElement(SimpleCellFormatter, { value: _this.props.value });
      }
      var isExpandCell = _this.props.expandableOptions ? _this.props.expandableOptions.field === _this.props.column.key : false;
      var treeDepth = _this.props.expandableOptions ? _this.props.expandableOptions.treeDepth : 0;
      var marginLeft = _this.props.expandableOptions && isExpandCell ? _this.props.expandableOptions.treeDepth * 30 : 0;
      var cellExpander = void 0;
      var cellDeleter = void 0;
      if (_this.canExpand()) {
        cellExpander = React.createElement(_CellExpand2['default'], { expandableOptions: _this.props.expandableOptions, onCellExpand: _this.onCellExpand });
      }

      var isDeleteSubRowEnabled = _this.props.cellMetaData.onDeleteSubRow ? true : false;

      if (treeDepth > 0 && isExpandCell) {
        cellDeleter = React.createElement(_ChildRowDeleteButton2['default'], { treeDepth: treeDepth, cellHeight: _this.props.height, siblingIndex: _this.props.expandableOptions.subRowDetails.siblingIndex, numberSiblings: _this.props.expandableOptions.subRowDetails.numberSiblings, onDeleteSubRow: _this.onDeleteSubRow, isDeleteSubRowEnabled: isDeleteSubRowEnabled });
      }
      return React.createElement(
        'div',
        { className: 'react-grid-Cell__value' },
        cellDeleter,
        React.createElement(
          'div',
          { style: { marginLeft: marginLeft } },
          React.createElement(
            'span',
            null,
            CellContent
          ),
          ' ',
          _this.props.cellControls,
          ' ',
          cellExpander
        )
      );
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(Cell, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.checkFocus();
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        isCellValueChanging: this.props.isCellValueChanging(this.props.value, nextProps.value),
        isLockChanging: this.props.column.locked !== nextProps.column.locked
      });
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.checkFocus();
      var dragged = this.props.cellMetaData.dragged;
      if (dragged && dragged.complete === true) {
        this.props.cellMetaData.handleTerminateDrag();
      }
      if (this.state.isCellValueChanging && this.props.selectedColumn != null) {
        this.applyUpdateClass();
      }
      if (this.state.isLockChanging && !this.props.column.locked) {
        this.removeScroll();
      }
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      var shouldUpdate = this.props.column.width !== nextProps.column.width || this.props.column.left !== nextProps.column.left || this.props.column.cellClass !== nextProps.column.cellClass || this.props.height !== nextProps.height || this.props.rowIdx !== nextProps.rowIdx || this.isCellSelectionChanging(nextProps) || this.isDraggedCellChanging(nextProps) || this.isCopyCellChanging(nextProps) || this.props.isRowSelected !== nextProps.isRowSelected || this.isSelected() || this.props.isCellValueChanging(this.props.value, nextProps.value) || this.props.forceUpdate === true || this.props.className !== nextProps.className || this.props.expandableOptions !== nextProps.expandableOptions || this.hasChangedDependentValues(nextProps) || this.props.column.locked !== nextProps.column.locked;
      return shouldUpdate;
    }
  }, {
    key: 'getCellActions',
    value: function getCellActions() {
      var _props = this.props,
          cellMetaData = _props.cellMetaData,
          column = _props.column,
          rowData = _props.rowData;

      if (cellMetaData && cellMetaData.getCellActions) {
        var cellActions = cellMetaData.getCellActions(column, rowData);
        if (cellActions && cellActions.length) {
          return cellActions.map(function (action, index) {
            return React.createElement(_CellAction2['default'], { key: index, action: action, isFirst: index === 0 });
          });
        }
        return null;
      }
      return null;
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      if (this.props.column.hidden) {
        return null;
      }

      var style = this.getStyle();

      var className = this.getCellClass();

      var cellActions = this.getCellActions();

      var cellContent = this.props.children || this.renderCellContent({
        value: this.props.value,
        column: this.props.column,
        rowIdx: this.props.rowIdx,
        isExpanded: this.props.isExpanded
      });

      var dragHandle = !this.isActive() && ColumnUtils.canEdit(this.props.column, this.props.rowData, this.props.cellMetaData.enableCellSelect) ? React.createElement(
        'div',
        { className: 'drag-handle', draggable: 'true', onDoubleClick: this.onDragHandleDoubleClick },
        React.createElement('span', { style: { display: 'none' } })
      ) : null;
      var events = this.getEvents();
      var tooltip = this.props.tooltip ? React.createElement(
        'span',
        { className: 'cell-tooltip-text' },
        this.props.tooltip
      ) : null;

      return React.createElement(
        'div',
        _extends({}, this.getKnownDivProps(), { className: className, style: style }, events, { ref: function ref(node) {
            _this2.node = node;
          } }),
        cellActions,
        cellContent,
        dragHandle,
        tooltip
      );
    }
  }]);

  return Cell;
}(React.Component), _class.defaultProps = {
  tabIndex: 0,
  isExpanded: false,
  value: '',
  isCellValueChanging: function isCellValueChanging(value, nextValue) {
    return value !== nextValue;
  }
}, _temp2);
Cell.propTypes = process.env.NODE_ENV !== "production" ? {
  rowIdx: _propTypes2['default'].number.isRequired,
  idx: _propTypes2['default'].number.isRequired,
  selected: _propTypes2['default'].shape({
    idx: _propTypes2['default'].number.isRequired
  }),
  selectedColumn: _propTypes2['default'].object,
  height: _propTypes2['default'].number,
  tabIndex: _propTypes2['default'].number,
  column: _propTypes2['default'].shape(ExcelColumn).isRequired,
  value: _propTypes2['default'].oneOfType([_propTypes2['default'].string, _propTypes2['default'].number, _propTypes2['default'].object, _propTypes2['default'].bool]),
  isExpanded: _propTypes2['default'].bool,
  isRowSelected: _propTypes2['default'].bool,
  cellMetaData: _propTypes2['default'].shape(CellMetaDataShape).isRequired,
  handleDragStart: _propTypes2['default'].func,
  className: _propTypes2['default'].string,
  cellControls: _propTypes2['default'].any,
  rowData: _propTypes2['default'].object.isRequired,
  forceUpdate: _propTypes2['default'].bool,
  expandableOptions: _propTypes2['default'].object.isRequired,
  isScrolling: _propTypes2['default'].bool.isRequired,
  tooltip: _propTypes2['default'].string,
  isCellValueChanging: _propTypes2['default'].func,
  children: _propTypes2['default'].oneOfType([_propTypes2['default'].arrayOf(_propTypes2['default'].node), _propTypes2['default'].node])
} : {};


module.exports = Cell;