'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _temp2;

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var React = require('react');
var shallowEqual = require('shallowequal');
var BaseHeaderCell = require('./HeaderCell');
var getScrollbarSize = require('./getScrollbarSize');
var ExcelColumn = require('./PropTypeShapes/ExcelColumn');
var columnUtils = require('./ColumnUtils');
var SortableHeaderCell = require('./cells/headerCells/SortableHeaderCell');
var FilterableHeaderCell = require('./cells/headerCells/FilterableHeaderCell');
var HeaderCellType = require('./HeaderCellType');
var SortDataShape = require('./PropTypeShapes/SortDataShape');
var createObjectWithProperties = require('./createObjectWithProperties');
var areSortArraysEqual = require('./utils/areSortArraysEqual');
require('./themes/react-data-grid-header.css');

var HeaderRowStyle = {
  overflow: _propTypes2['default'].string,
  width: _propTypes2['default'].oneOfType([_propTypes2['default'].number, _propTypes2['default'].string]),
  height: _propTypes2['default'].number,
  position: _propTypes2['default'].string
};

// The list of the propTypes that we want to include in the HeaderRow div
var knownDivPropertyKeys = ['width', 'height', 'style', 'onScroll'];

var HeaderRow = (_temp2 = _class = function (_React$Component) {
  _inherits(HeaderRow, _React$Component);

  function HeaderRow() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, HeaderRow);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = HeaderRow.__proto__ || Object.getPrototypeOf(HeaderRow)).call.apply(_ref, [this].concat(args))), _this), _this.getHeaderCellType = function (column) {
      if (column.filterable) {
        if (_this.props.filterable) return HeaderCellType.FILTERABLE;
      }

      if (column.sortable && column.rowType !== 'filter') return HeaderCellType.SORTABLE;

      return HeaderCellType.NONE;
    }, _this.getFilterableHeaderCell = function (column) {
      var FilterRenderer = FilterableHeaderCell;
      if (column.filterRenderer !== undefined) {
        FilterRenderer = column.filterRenderer;
      }
      return React.createElement(FilterRenderer, _extends({}, _this.props, { onChange: _this.props.onFilterChange }));
    }, _this.getSortableHeaderCell = function (column) {
      var sortDirection = void 0;
      var sortDescendingFirst = column.sortDescendingFirst === undefined ? false : column.sortDescendingFirst;
      if (_this.props.sort) {
        var columnSort = _this.props.sort.filter(function (s) {
          return s.column === column.key;
        })[0];
        sortDirection = columnSort ? columnSort.direction : SortableHeaderCell.DEFINE_SORT.NONE;
      } else {
        sortDirection = _this.props.sortColumn === column.key ? _this.props.sortDirection : SortableHeaderCell.DEFINE_SORT.NONE;
      }
      return React.createElement(SortableHeaderCell, { columnKey: column.key, onSort: _this.props.onSort, sortDirection: sortDirection,
        sortDescendingFirst: sortDescendingFirst, headerRenderer: column.headerRenderer });
    }, _this.getHeaderRenderer = function (column) {
      var renderer = void 0;
      var canSort = false;
      if (_this.props.appliedFilters) {
        for (var i = 0, len = columnUtils.getSize(_this.props.columns); i < len; i++) {
          var _column = Object.assign({ rowType: _this.props.rowType }, columnUtils.getColumn(_this.props.columns, i));
          if (_column.filterFirst === true && _this.props.appliedFilters[_column.key] != undefined && _this.props.appliedFilters[_column.key] != '') {
            canSort = true;
          }
        }
      }
      if (canSort === false) {
        return renderer;
      }
      if (column.headerRenderer && !column.sortable && !_this.props.filterable) {
        renderer = column.headerRenderer;
      } else {
        var headerCellType = _this.getHeaderCellType(column);

        switch (headerCellType) {
          case HeaderCellType.SORTABLE:
            renderer = _this.getSortableHeaderCell(column);
            break;
          case HeaderCellType.FILTERABLE:
            renderer = _this.getFilterableHeaderCell(column);
            break;
          default:
            break;
        }
      }
      return renderer;
    }, _this.getStyle = function () {
      return {
        overflow: 'hidden',
        width: '100%',
        height: _this.props.height,
        position: 'absolute'
      };
    }, _this.getCells = function () {
      var cells = [];
      var lockedCells = [];
      //find exists required filter
      var canFilter = false;
      if (_this.props.appliedFilters) {
        for (var i = 0, len = columnUtils.getSize(_this.props.columns); i < len; i++) {
          var column = Object.assign({ rowType: _this.props.rowType }, columnUtils.getColumn(_this.props.columns, i));
          if (column.filterFirst === true && _this.props.appliedFilters[column.key] != undefined && _this.props.appliedFilters[column.key] != '') {
            canFilter = true;
          }
        }
      }

      var _loop = function _loop(_i, _len2) {
        var column = Object.assign({ rowType: _this.props.rowType }, columnUtils.getColumn(_this.props.columns, _i));
        var _renderer = _this.getHeaderRenderer(column);
        if (column.key === 'select-row' && _this.props.rowType === 'filter') {
          _renderer = React.createElement('div', null);
        }
        var isFilterSet = false;
        var filterValue = '';
        if (_this.props.appliedFilters) {
          if (_this.props.appliedFilters[column.key] != undefined && _this.props.appliedFilters[column.key] != '') {
            isFilterSet = true;
            filterValue = _this.props.appliedFilters[column.key];
          }
        }
        var HeaderCell = column.draggable ? _this.props.draggableHeaderCell : BaseHeaderCell;
        var cell = React.createElement(HeaderCell, {
          ref: function ref(node) {
            return _this.cells[_i] = node;
          },
          key: _i,
          height: _this.props.height,
          column: column,
          renderer: _renderer,
          resizing: _this.props.resizing === column,
          onResize: _this.props.onColumnResize,
          onResizeEnd: _this.props.onColumnResizeEnd,
          onHeaderDrop: _this.props.onHeaderDrop,
          canFilter: canFilter,
          isFilterSet: isFilterSet,
          filterValue: filterValue,
          showFilterCallback: _this.props.showFilterCallback
        });
        if (column.locked) {
          lockedCells.push(cell);
        } else {
          cells.push(cell);
        }
      };

      for (var _i = 0, _len2 = columnUtils.getSize(_this.props.columns); _i < _len2; _i++) {
        _loop(_i, _len2);
      }

      return cells.concat(lockedCells);
    }, _this.setScrollLeft = function (scrollLeft) {
      _this.props.columns.forEach(function (column, i) {
        if (column.locked) {
          _this.cells[i].setScrollLeft(scrollLeft);
        } else {
          if (_this.cells[i] && _this.cells[i].removeScroll) {
            _this.cells[i].removeScroll();
          }
        }
      });
    }, _this.getKnownDivProps = function () {
      return createObjectWithProperties(_this.props, knownDivPropertyKeys);
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(HeaderRow, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      this.cells = [];
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      return nextProps.width !== this.props.width || nextProps.height !== this.props.height || nextProps.columns !== this.props.columns || nextProps.appliedFilters !== this.props.appliedFilters || !shallowEqual(nextProps.style, this.props.style) || !areSortArraysEqual(nextProps.sort, this.props.sort) || this.props.sortColumn !== nextProps.sortColumn || this.props.sortDirection !== nextProps.sortDirection;
    }
  }, {
    key: 'render',
    value: function render() {
      var cellsStyle = {
        width: this.props.width ? this.props.width + getScrollbarSize() : '100%',
        height: this.props.height,
        whiteSpace: 'nowrap',
        overflowX: 'hidden',
        overflowY: 'hidden'
      };

      var cells = this.getCells();
      return React.createElement(
        'div',
        _extends({}, this.getKnownDivProps(), { className: 'react-grid-HeaderRow' }),
        React.createElement(
          'div',
          { style: cellsStyle },
          React.createElement(
            'div',
            { style: { position: 'absolute', width: this.props.width ? this.props.width + getScrollbarSize() : '100%', height: this.props.height }, className: 'react-grid-HeaderRow__wrapper' },
            cells
          )
        )
      );
    }
  }]);

  return HeaderRow;
}(React.Component), _class.displayName = 'HeaderRow', _temp2);
HeaderRow.propTypes = process.env.NODE_ENV !== "production" ? {
  width: _propTypes2['default'].oneOfType([_propTypes2['default'].number, _propTypes2['default'].string]),
  height: _propTypes2['default'].number.isRequired,
  columns: _propTypes2['default'].oneOfType([_propTypes2['default'].array, _propTypes2['default'].object]).isRequired,
  onColumnResize: _propTypes2['default'].func,
  onSort: _propTypes2['default'].func.isRequired,
  onColumnResizeEnd: _propTypes2['default'].func,
  style: _propTypes2['default'].shape(HeaderRowStyle),
  sortColumn: _propTypes2['default'].string,
  sortDirection: _propTypes2['default'].oneOf(Object.keys(SortableHeaderCell.DEFINE_SORT)),
  sort: SortDataShape,
  cellRenderer: _propTypes2['default'].func,
  headerCellRenderer: _propTypes2['default'].func,
  filterable: _propTypes2['default'].bool,
  onFilterChange: _propTypes2['default'].func,
  resizing: _propTypes2['default'].object,
  onScroll: _propTypes2['default'].func,
  rowType: _propTypes2['default'].string,
  draggableHeaderCell: _propTypes2['default'].func,
  onHeaderDrop: _propTypes2['default'].func,
  //{"owner":"jsfja"}
  appliedFilters: _propTypes2['default'].object,
  showFilterCallback: _propTypes2['default'].func.isRequired
} : {};


module.exports = HeaderRow;