'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getRenderedColumnCount = exports.getNextScrollState = exports.getGridState = undefined;

var _ColumnUtils = require('../ColumnUtils');

var _ColumnUtils2 = _interopRequireDefault(_ColumnUtils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var min = Math.min;
var max = Math.max;
var floor = Math.floor;
var ceil = Math.ceil;

function getGridState(props) {
  var totalNumberColumns = _ColumnUtils2['default'].getSize(props.columnMetrics.columns);
  var canvasHeight = props.minHeight - props.rowOffsetHeight;
  var renderedRowsCount = ceil((props.minHeight - props.rowHeight) / props.rowHeight);
  var totalRowCount = min(renderedRowsCount * 4, props.rowsCount);
  return {
    displayStart: 0,
    displayEnd: totalRowCount,
    visibleStart: 0,
    visibleEnd: totalRowCount,
    height: canvasHeight,
    scrollTop: 0,
    scrollLeft: 0,
    colVisibleStart: 0,
    colVisibleEnd: totalNumberColumns,
    colDisplayStart: 0,
    colDisplayEnd: totalNumberColumns
  };
}

function getRenderedColumnCount(props, getDOMNodeOffsetWidth, displayStart, width) {
  var remainingWidth = width && width > 0 ? width : props.columnMetrics.totalWidth;
  if (remainingWidth === 0) {
    remainingWidth = getDOMNodeOffsetWidth();
  }
  var columnIndex = displayStart;
  var columnCount = 0;
  while (remainingWidth > 0) {
    var column = _ColumnUtils2['default'].getColumn(props.columnMetrics.columns, columnIndex);

    if (!column) {
      break;
    }

    columnCount++;
    columnIndex++;
    remainingWidth -= column.width;
  }
  return columnCount;
}

function getVisibleColStart(props, scrollLeft) {
  var remainingScroll = scrollLeft;
  var columnIndex = -1;
  while (remainingScroll >= 0) {
    columnIndex++;
    remainingScroll -= _ColumnUtils2['default'].getColumn(props.columnMetrics.columns, columnIndex).width;
  }
  return columnIndex;
}

function getNextScrollState(props, getDOMNodeOffsetWidth, scrollTop, scrollLeft, height, rowHeight, length, width) {
  var isScrolling = true;
  var renderedRowsCount = ceil(height / rowHeight);
  var visibleStart = max(0, floor(scrollTop / rowHeight));
  var visibleEnd = min(visibleStart + renderedRowsCount, length);
  var displayStart = max(0, visibleStart - props.overScan.rowsStart);
  var displayEnd = min(visibleEnd + props.overScan.rowsEnd, length);
  var totalNumberColumns = _ColumnUtils2['default'].getSize(props.columnMetrics.columns);
  var colVisibleStart = totalNumberColumns > 0 ? max(0, getVisibleColStart(props, scrollLeft)) : 0;
  var renderedColumnCount = getRenderedColumnCount(props, getDOMNodeOffsetWidth, colVisibleStart, width);
  var colVisibleEnd = renderedColumnCount !== 0 ? colVisibleStart + renderedColumnCount : totalNumberColumns;
  var colDisplayStart = max(0, colVisibleStart - props.overScan.colsStart);
  var colDisplayEnd = min(colVisibleEnd + props.overScan.colsEnd, totalNumberColumns);

  var nextScrollState = {
    visibleStart: visibleStart,
    visibleEnd: visibleEnd,
    displayStart: displayStart,
    displayEnd: displayEnd,
    height: height,
    scrollTop: scrollTop,
    scrollLeft: scrollLeft,
    colVisibleStart: colVisibleStart,
    colVisibleEnd: colVisibleEnd,
    colDisplayStart: colDisplayStart,
    colDisplayEnd: colDisplayEnd,
    isScrolling: isScrolling
  };

  return nextScrollState;
}

exports.getGridState = getGridState;
exports.getNextScrollState = getNextScrollState;
exports.getRenderedColumnCount = getRenderedColumnCount;