import React, {Component, Fragment} from 'react';

const LEFT_PAGE = 'LEFT';
const RIGHT_PAGE = 'RIGHT';
const ELLIPSIS_PAGE = 'ELLIPSIS';

const range = (from, to, step = 1) => {
  let i = from;
  const range = [];

  while (i <= to) {
    range.push(i);
    i += step;
  }

  return range;
};

class Pagination extends Component {
  pageLimit;
  totalPages;
  pageNeighbours;
  ellipsis;
  currentPage;

  constructor(props) {
    super(props);
    const {pageLimit = 30, pageNeighbours = 0, ellipsis = false} = props;

    this.ellipsis = ellipsis == null ? false : ellipsis;
    this.pageLimit = typeof pageLimit === 'number' ? pageLimit : 30;

    this.pageNeighbours = typeof pageNeighbours === 'number' ? Math.max(0, pageNeighbours) : 0;

    this.totalPages = this.props.totalPages < 10 ? this.props.totalPages - 1 : this.props.totalPages;

    this.state = {currentPage: 1};
  }

  componentDidMount() {
    if(isNaN(this.props.currentPage)){
      this.gotoPage(1);
    }
    else {
      this.gotoPage(this.props.currentPage);
    }
  }

  gotoPage = page => {
    const {onPageChanged = f => f} = this.props;

    const currentPage = Math.max(0, Math.min(page, this.props.totalPages));

    const paginationData = {
      currentPage,
      totalPages: this.props.totalPages < 10 ? this.props.totalPages - 1 : this.props.totalPages,
      pageLimit: this.pageLimit,
    };

    this.setState({currentPage}, () => onPageChanged(paginationData));
    console.log(paginationData)
  };

  handleClick = (page, evt) => {
    evt.preventDefault();
    this.gotoPage(page);
  };

  handleMoveLeft = evt => {
    evt.preventDefault();
    this.gotoPage(this.state.currentPage - 1);
  };

  handleMoveRight = evt => {
    evt.preventDefault();
    this.gotoPage(this.state.currentPage + 1);
  };

  _endPage = 10;
  handleMoveMoreRight = evt => {
    evt.preventDefault();
    this.gotoPage(this._endPage + 1);
  };

  fetchPageNumbers = () => {
    const totalPages = this.props.totalPages < 10 ? this.props.totalPages - 1 : this.props.totalPages;
    const currentPage = this.state.currentPage;
    const pageNeighbours = this.pageNeighbours;

    const totalNumbers = this.pageNeighbours * 2 + 2;
    const totalBlocks = totalNumbers + 3;

    if (totalPages > totalBlocks) {
      let pages = [];

      const leftBound = currentPage - pageNeighbours;
      const rightBound = currentPage + pageNeighbours;
      const beforeLastPage = totalPages - 1;

      const startPage = leftBound > 2 ? leftBound : 2;
      const endPage = rightBound < beforeLastPage ? rightBound : beforeLastPage;

      pages = range(startPage, endPage);

      const pagesCount = pages.length;
      const singleSpillOffset = totalNumbers - pagesCount - 1;

      const leftSpill = startPage > 3;
      if (startPage === 3)
        pages = [2, ...pages];
      const rightSpill = endPage < beforeLastPage;

      const leftSpillPage = LEFT_PAGE;
      const rightSpillPage = RIGHT_PAGE;
      const ellipsisSpillPage = ELLIPSIS_PAGE;

      if (leftSpill && !rightSpill) {
        const extraPages = range(startPage - singleSpillOffset, startPage - 1);
        pages = [leftSpillPage, ...extraPages, ...pages];
      } else if (!leftSpill && rightSpill) {
        const extraPages = range(endPage + 1, endPage + singleSpillOffset);
        this._endPage = extraPages[extraPages.length - 1] || 10;
        pages = [...pages, ...extraPages, rightSpillPage];
      } else if (leftSpill && rightSpill) {
        this._endPage = pages[pages.length - 1] || 10;
        pages = [leftSpillPage, ...pages, rightSpillPage];
      }

      if (!this.ellipsis) {
        return [1, ...pages, totalPages];
      }
      else {
          if(rightSpill) {
              return [1, ...pages, ellipsisSpillPage];
          }
          else{
              return [1, ...pages];
          }
      }
    }
    return range(1, totalPages);
  };

  render() {
    const {currentPage} = this.state;
    const pages = this.fetchPageNumbers();

    return (
        <Fragment>
            <nav style={{display: 'inline-block', marginLeft: '200px'}} aria-label="Countries Pagination">
                <ul className="pagination" style={{width: 'auto'}}>
                    {pages.map((page, index) => {
                        if (page === LEFT_PAGE) {
                            return (
                                <li key={index} className="page-item">
                                    <a style={{height: '100%'}} className="page-link" href="#" aria-label="Previous" onClick={this.handleMoveLeft}>
                        <span style={{
                            border: 'solid black',
                            borderWidth: '0 2px 2px 0',
                            display: 'inline-block',
                            padding: '3px',
                            position: 'relative',
                            top: '-1px',
                            left: '2px',
                            transform: 'rotate(135deg)'}}    aria-hidden="true"></span>
                                        <span className="sr-only">Previous</span>
                                    </a>
                                </li>
                            );
                        }

                        if (page === RIGHT_PAGE) {
                            return (
                                <li key={index} className="page-item">
                                    <a style={{height: '100%'}}   className="page-link" href="#" aria-label="Next" onClick={this.handleMoveRight}>
                        <span style={{
                            border: 'solid black',
                            borderWidth: '0 2px 2px 0',
                            display: 'inline-block',
                            padding: '3px',
                            position: 'relative',
                            top: '-1px',
                            left: '-1px',
                            transform: 'rotate(-45deg)'}} aria-hidden="true"></span>
                                        <span className="sr-only">Next</span>
                                    </a>
                                </li>
                            );
                        }

                        if (page === ELLIPSIS_PAGE) {
                            return (
                                <li key={index} className="page-item">
                                    <a className="page-link" href="#" aria-label="More" onClick={this.handleMoveMoreRight}>
                                        <span aria-hidden="true">...</span>
                                        <span className="sr-only">More</span>
                                    </a>
                                </li>
                            );
                        }

                        return (
                            <li key={index} className={`page-item${currentPage === page ? ' active' : ''}`}>
                                <a className="page-link" href="#" onClick={e => this.handleClick(page, e)}>
                                    {page}
                                </a>
                            </li>
                        );
                    })}
                </ul>
            </nav>
        </Fragment>
    );
  }
}

export default Pagination;
