import React from 'react';
import PropTypes from 'prop-types';

export const DEFAULT_CONTEXT_MENU_ID = 'rgdContextMenu';

const SimpleRowsContainer = (props) => <div key="rows-container" style={{width: props.width}}>{props.rows}</div>;

SimpleRowsContainer.propTypes = {
    width: PropTypes.number,
    rows: PropTypes.array
};

export const getNewContextMenuProps = ({contextMenu, rowIdx, idx}) => ({
    rowIdx, idx, id: contextMenu.props.id || DEFAULT_CONTEXT_MENU_ID
});

class RowsContainer extends React.Component {
    constructor(props) {
        super(props);
        this.plugins = props.window ? props.window.ReactDataGridPlugins : window.ReactDataGridPlugins;
    }

    validatePlugin() {
        if (!this.plugins) {
            throw new Error('You need to include ReactDataGrid UiPlugins in order to initialise context menu');
        }
    }

    hasContextMenu() {
        return this.props.contextMenu && React.isValidElement(this.props.contextMenu);
    }

    renderRowsWithContextMenu() {
        const {ContextMenuTrigger} = this.plugins.Menu;
        const newProps = getNewContextMenuProps(this.props);
        const contextMenu = React.cloneElement(this.props.contextMenu, newProps);
        // Initialise the context menu if it is available
        return (
            <div>
                <ContextMenuTrigger id={newProps.id}>
                    <div key="rows-container" style={{width: this.props.width}}>{this.props.rows}</div>
                </ContextMenuTrigger>
                {contextMenu}
            </div>
        );
    }

    render() {
        if (this.hasContextMenu()) {
            this.validatePlugin();
            return this.renderRowsWithContextMenu();
        }

        return (<div key="rows-container" style={{width: this.props.width, height: '100px'}}>
            {/*<div style={{position:'absolute', zIndex:100}}>{this.props.columns.map((r, idx) => {
        if (r.filterComponent){
          return r.filterComponent(idx, r);
        }
      })}
      </div>*/}
            {this.props.rows}
        </div>);
    }
}

RowsContainer.propTypes = {
    contextMenu: PropTypes.element,
    rowIdx: PropTypes.number,
    idx: PropTypes.number,
    window: PropTypes.object,
    columns: PropTypes.oneOfType([PropTypes.object, PropTypes.array]).isRequired
};

export default RowsContainer;
export {SimpleRowsContainer};
